// PV227, Lesson 11: Image Based Lighting
#pragma once

#include "../../Framework/PV227.h"

using namespace std;
using namespace PV227;

//---------------------
//----    SCENE    ----
//---------------------

// Shader programs
ShaderProgram depth_prepass_program;
ShaderProgram skybox_program;
ShaderProgram object_program;

// Geometries we use in this lecture
Geometry geom_cube;
Geometry geom_sphere;
Geometry geom_torus;
Geometry geom_cylinder;
Geometry geom_capsule;
Geometry geom_teapot;
Geometry geom_fullscreen_quad;

// A camera that allows us to look at the object from different views
SimpleCamera the_camera;

// Textures
GLuint mask_tex;
GLuint meadow_skybox_tex;
GLuint miramar_skybox_tex;

GLuint current_skybox_tex;

// Positions of objects
ModelData_UBO CentralObjects_ubo[64];

// Materials of objects
glm::vec3 material_diffuses[64];
glm::vec3 material_fresnels[64];
float material_roughnesses[64];

// UBO with random floats for sampling
GLuint RandomFloats_UBO;

// Data of our camera - view matrix, projection matrix, etc.
CameraData_UBO CameraData_ubo;

// OpenGL query object to get render time of one frame
GLuint RenderTimeQuery;

// Functions that works with scene objects
void reload_shaders();
void init_scene();
void update_scene(int app_time_diff_ms);
void render_scene();

//-------------------
//----    GUI    ----
//-------------------

// Main AntTweakBar object
TwBar *the_gui;

// Variables that are changed with GUI
float render_time_ms;		// Read-only

int skybox_texture;
const int SKYBOX_MEADOW = 0;
const int SKYBOX_MIRAMAR = 1;

int central_object;
const int CENTRAL_OBJECT_CUBE = 0;
const int CENTRAL_OBJECT_SPHERE = 1;
const int CENTRAL_OBJECT_TORUS = 2;
const int CENTRAL_OBJECT_CYLINDER = 3;
const int CENTRAL_OBJECT_CAPSULE = 4;
const int CENTRAL_OBJECT_TEAPOT = 5;

int diffuse_sample_count;
int specular_sample_count;
bool mask_roughness;
bool use_shiny_layer;
bool use_shiny_layer_mask;

// Callbacks from the GUI
void TW_CALL reload(void *);

// Functions that works with GUI
void init_gui();

//---------------------------
//----    APPLICATION    ----

// Current window size
int win_width = 640;
int win_height = 480;

// Current time of the application in milliseconds, for animations
int app_time_ms = 0;
int last_glut_time = 0;
